/*
 * Plot - plot time-based data on screen and to file with interactive controls
 * Copyright (C) 2006  Jonas Berlin <xkr47@outerspace.dyndns.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****
 * eventhandlers-plot-keyboard - gtk event handlers related handling
 * keyboard events for the plot
 */

#include <gtk/gtktogglebutton.h>
#include <gdk/gdkkeysyms.h>

#include "eventhandlers-plot-keyboard.xh"
#include "plot2-zoom-history.xh"

#if 0
#define pprintf printf
#else
#define pprintf(...)
#endif


static void perform_arrow(GdkEventKey *event, my_plot_view_t *view, int is_y, int dir) {
/*
   if(event->state & GDK_SHIFT_MASK) {
      dir *= 5;
   }
*/
   data_scrollbar_controller_t *dsc = is_y ? view->dsc_y : view->parent->dsc_x;

   if(event->state & GDK_SHIFT_MASK) {
      // zoom
      data_scrollbar_controller_set_page_length_log(dsc,
						    data_scrollbar_controller_get_page_length_log(dsc) +
						    (is_y ? -1 : 1) * dir / 10.0);

   } else {
      // move

#ifdef USE_FAM
      boolean_t tailing = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(view->parent->checkbutton_tail));
#else
      const boolean_t tailing = 0;
#endif
      if(!tailing || is_y) {
	 data_scrollbar_controller_set_page_off(dsc,
						data_scrollbar_controller_get_page_off(dsc) -
						data_scrollbar_controller_get_page_length(dsc) * dir / 20.0);
      }
   }
}

gboolean plot_key_event(GtkWidget *widget, GdkEventKey *event, my_plot_view_t *view) {

   pprintf("%d: %s %sstate 0%o kv %d text '%.*s' hkc %d grp %d\n",
	   event->time,
	   event->type == GDK_KEY_PRESS ? "PRESS  " : "RELEASE",
	   event->send_event ? "se " : "",
	   event->state,
	   event->keyval,
	   event->length,
	   event->string,
	   event->hardware_keycode,
	   event->group);

   switch(event->type) {
      case GDK_KEY_PRESS:
	 switch(event->keyval) {
	    case GDK_Control_L:
	    case GDK_Control_R:
	       view->parent->ctrl_pressed = 1;
	       update_cursor(view);
	       return 1;
	       
	    case GDK_space:
	       show_all(view);
	       return 1;
	       
	    case GDK_KP_Home:
	    case GDK_KP_7:
	       perform_arrow(event, view, 0, 1);
	       perform_arrow(event, view, 1, -1);
	       delayed_add_to_zoom_history(view->parent);
	       return 1;

	    case GDK_Up:
	    case GDK_KP_Up:
	    case GDK_KP_8:
	       perform_arrow(event, view, 1, -1);
	       delayed_add_to_zoom_history(view->parent);
	       return 1;

	    case GDK_KP_Page_Up:
	    case GDK_KP_9:
	       perform_arrow(event, view, 0, -1);
	       perform_arrow(event, view, 1, -1);
	       delayed_add_to_zoom_history(view->parent);
	       return 1;

	    case GDK_Left:
	    case GDK_KP_Left:
	    case GDK_KP_4:
	       perform_arrow(event, view, 0, 1);
	       delayed_add_to_zoom_history(view->parent);
	       return 1;

	    case GDK_Right:
	    case GDK_KP_Right:
	    case GDK_KP_6:
	       perform_arrow(event, view, 0, -1);
	       delayed_add_to_zoom_history(view->parent);
	       return 1;

	    case GDK_KP_End:
	    case GDK_KP_1:
	       perform_arrow(event, view, 0, 1);
	       perform_arrow(event, view, 1, 1);
	       delayed_add_to_zoom_history(view->parent);
	       return 1;

	    case GDK_Down:
	    case GDK_KP_Down:
	    case GDK_KP_2:
	       perform_arrow(event, view, 1, 1);
	       delayed_add_to_zoom_history(view->parent);
	       return 1;

	    case GDK_KP_Page_Down:
	    case GDK_KP_3:
	       perform_arrow(event, view, 0, -1);
	       perform_arrow(event, view, 1, 1);
	       delayed_add_to_zoom_history(view->parent);
	       return 1;
	 }
	 break;

      case GDK_KEY_RELEASE:
	 switch(event->keyval) {
	    case GDK_Control_L:
	    case GDK_Control_R:
	       view->parent->ctrl_pressed = 0;
	       update_cursor(view);
	       return 1;
	 }
	 break;

      default:
	 fprintf(stderr, "Unknown keyboard event type %d\n", event->type);
	 break;
   }
   
   return 0;
}

/*
 * Local variables:
 * c-file-style: "ellemtel"
 * c-file-offsets: ((c . c-lineup-dont-change) (statement-cont . (lambda (le) (if (save-excursion (goto-char (cdr le)) (looking-at "return")) (c-lineup-java-inher le) (c-lineup-math le)))))
 * End:
 */
