/*
 * Plot - plot time-based data on screen and to file with interactive controls
 * Copyright (C) 2006  Jonas Berlin <xkr47@outerspace.dyndns.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****
 * main - main function setting up some gtk stuff and loading data
 * files given on command line
 */

#include <glade/glade-xml.h>
#include <gtk/gtkmain.h>
#include <gtk/gtkversion.h>

#include "../mem.xh"
#include "../system.xh"
#include "eventhandlers.xh"
#include "plot2-datafiles.xh"
#include "plot2.xh"

boolean_t quit = 0;

boolean_t system_check_events(void) {
   while(gtk_events_pending()) {
      gtk_main_iteration();
   }

   if(quit) {
      if(gtk_main_level()) gtk_main_quit();
   }

   return quit;
}

boolean_t request_quit(void) {
   if(gtk_main_level()) gtk_main_quit();
   quit = 1;
   return 1;
}

char *filename_to_utf8(const char *filename) {
#if !GTK_CHECK_VERSION(2,6,0)
   GError *error = NULL;
   char *file_utf8 = g_filename_to_utf8(filename, (gssize)-1, NULL, NULL, &error);
   if(!file_utf8) {
      fprintf(stderr, "g_filename_to_utf8() error for '%s': %s\n", filename, error->message);
      g_error_free(error);
      return strdup("<unable to show filename>");
   }
   return file_utf8;
#else
   return g_filename_display_name(filename);
#endif
}

#define setcolor(x,r_,g_,b_) x.r=r_;x.g=g_;x.b=b_;

int main(int arg_count, char **arg) {
#ifdef ENABLE_NLS
   bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
   bind_textdomain_codeset(PACKAGE, "UTF-8");
   textdomain(PACKAGE);
#endif

   gtk_init(&arg_count, &arg);

#ifdef GLADEFILE
   const char *gladefile = GLADEFILE;
#else
   const char *gladefile = "plot.glade";
   char *lpp = strrchr(arg[0], '/');
   if(lpp) {
      size_t lplen = lpp-arg[0] + 1;
      size_t filelen = strlen(gladefile);
      char *new = malloc(lplen + filelen + 1U);
      memcpy(new, arg[0], lplen);
      memcpy(new + lplen, gladefile, filelen + 1);
      gladefile = new;
   }
   printf(_c("Using glade file %s\n"), gladefile);
#endif

   install_default_line_colors_as_gtk_palette();

   GladeXML *gladexml = glade_xml_new(gladefile, "mainwindow", NULL);

   GtkWidget *win = glade_xml_get_widget(gladexml, "mainwindow");

   g_signal_connect(win, "destroy-event", G_CALLBACK(request_quit), NULL);
   g_signal_connect(win, "delete-event", G_CALLBACK(request_quit), NULL);

   my_plot_t *plot = malloc(sizeof(my_plot_t));
   plot_init(plot);

   plot->dsc_x = data_scrollbar_controller_new(gladexml, "xscroll_", 1);

   plot_add_view_with_xml(plot, gladexml);

   plot->parent = GTK_WINDOW(win);
   plot->gladefile = gladefile;

   plot->view[0]->draw.show_disabled = 1;
   plot->font = g_strdup("Verdana 18.3");
   plot->fontsize_at_width = 1600;
   plot->scale_font_by_size = 1;
   plot->double_font_size = 0;
   plot->view[0]->draw.legend.show = LEGEND_SHOW_LEFT;

   plot->button_zoom_prev = glade_xml_get_widget(gladexml, "button_zoom_prev");
   plot->button_zoom_next = glade_xml_get_widget(gladexml, "button_zoom_next");
   gtk_widget_set_sensitive(plot->button_zoom_prev, 0);
   gtk_widget_set_sensitive(plot->button_zoom_next, 0);

   setcolor(plot->view[0]->draw.grid.minor_color, 0x6060, 0x6060, 0x6060);
   setcolor(plot->view[0]->draw.grid.major_color, 0xc0c0, 0xc0c0, 0xc0c0);
   setcolor(plot->view[0]->draw.grid.zerogrid_color, 0xffff, 0x6060, 0x4040);
   setcolor(plot->view[0]->draw.grid.sep_color, 0x8080, 0x8080, 0x8080);
   setcolor(plot->view[0]->draw.grid.label_color, 0xffff, 0xffff, 0xffff);
   setcolor(plot->cursor_marker_color, 0x8080, 0x8080, 0x8080);
   setcolor(plot->view[0]->draw.legend.bg_color, 0x2020, 0x2020, 0x2020);
   setcolor(plot->view[0]->draw.legend.border_color, 0xa0a0, 0xa0a0, 0xa0a0);
   setcolor(plot->view[0]->draw.legend.text_color, 0xe0e0, 0xe0e0, 0xe0e0);
   setcolor(plot->view[0]->draw.bookmarks.color, 0xffff, 0x9090, 0x5050);
   setcolor(plot->view[0]->draw.bookmarks.text_color, 0x1010, 0x1111, 0x1111);

   plot_connect_signals(plot, gladexml);

   plot->progressbar = GTK_PROGRESS_BAR(glade_xml_get_widget(gladexml, "progressbar"));
   plot->menuitem_file_add_data_file = glade_xml_get_widget(gladexml, "menuitem_file_add_data_file");

   if(system_check_events()) return 0;

   for(int a=1; a<arg_count; ++a) {
      add_datafile(plot, arg[a]);
   }

   if(!quit) gtk_main();

   plot_free(plot);

   return 0;
}

/*
 * Local variables:
 * c-file-style: "ellemtel"
 * c-file-offsets: ((c . c-lineup-dont-change) (statement-cont . (lambda (le) (if (save-excursion (goto-char (cdr le)) (looking-at "return")) (c-lineup-java-inher le) (c-lineup-math le)))))
 * End:
 */
