/*
 * Plot - plot time-based data on screen and to file with interactive controls
 * Copyright (C) 2006  Jonas Berlin <xkr47@outerspace.dyndns.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****
 * paintapi-test-gtk - main program for testing the paintapi-gtk with paintapi-test
 */

#include <gtk/gtkmain.h>
#include <gtk/gtkwidget.h>
#include <gtk/gtkdrawingarea.h>
#include <gtk/gtkwindow.h>

#include "../paintapi-test.xh"
#include "../plot_draw.xh"
#include "paintapi-gtk.xh"

boolean_t quit = 0;

GtkWidget *da;
my_plot_draw_t pd;

static gboolean map_event(GtkWidget *widget, GdkEvent *do_not_use, void *foo) {
   printf("Allocating paintapi\n");
   pd.paintapi = paintapi_gtk_new(da->window, &pd);
   return 1;
}

#define ZOOM 1

#define FONT "Verdana"
//#define FONT "Tahoma"
//#define FONT "Arial"
//#define FONT "Courier New"
//#define FONT "Lucida"

static gboolean expose_event(GtkWidget *widget, GdkEventExpose *event, void *foo) {
   int w,h;
   gdk_drawable_get_size(da->window, &w, &h);

   int wz = w / ZOOM;
   int hz = h / ZOOM;
   printf("Drawable size %dx%d - ", w, h);
   printf("Painting size %dx%d\n", wz, hz);

   paintapi_font_t font10;
   font10.ctx = gtk_widget_get_pango_context(da);
   font10.font = pango_font_description_from_string(FONT " 10");

   paintapi_font_t font20;
   font20.ctx = font10.ctx;
   font20.font = pango_font_description_from_string(FONT " 20");

   // perform test
   paintapi_test(pd.paintapi, &font10, &font20, wz-1, hz-1);

#if ZOOM > 1
   // scale image
   GdkPixbuf *p2 = gdk_pixbuf_get_from_drawable(NULL, da->window, NULL, 0, 0, 0, 0, wz, hz);
   GdkPixbuf *p = gdk_pixbuf_new(GDK_COLORSPACE_RGB, 0, 8, wz * ZOOM, hz * ZOOM);

   gdk_pixbuf_scale(p2, p, 0, 0, wz*ZOOM, hz*ZOOM, 0, 0, ZOOM, ZOOM, GDK_INTERP_NEAREST);

   GdkGC *gc = gdk_gc_new(da->window);

   gdk_pixbuf_render_to_drawable(p, da->window, gc, 0, 0, 0, 0, wz*ZOOM, hz*ZOOM, GDK_RGB_DITHER_NONE, 0, 0);

   gdk_gc_unref(gc);
#endif

   return 1;
}

static gboolean request_quit(void) {
   if(gtk_main_level()) gtk_main_quit();
   quit = 1;
   return 1;
}

int main(int arg_count, char **arg) {
   gtk_init(&arg_count, &arg);

   GtkWidget *win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_default_size(GTK_WINDOW(win), 460*ZOOM, 380*ZOOM);

   g_signal_connect(win, "destroy-event", G_CALLBACK(request_quit), NULL);
   g_signal_connect(win, "delete-event", G_CALLBACK(request_quit), NULL);

   da = gtk_drawing_area_new();
   gtk_container_add(GTK_CONTAINER(win), da);
   gtk_widget_show(da);

   g_signal_connect(da, "map-event", G_CALLBACK(map_event), NULL);
   g_signal_connect(da, "expose-event", G_CALLBACK(expose_event), NULL);

   GdkGeometry geometry;
   bzero(&geometry, sizeof(geometry));
   geometry.base_width = 0;
   geometry.base_height = 0;
   geometry.min_width = ZOOM;
   geometry.min_height = ZOOM;
   geometry.width_inc = ZOOM;
   geometry.height_inc = ZOOM;
   gtk_window_set_geometry_hints(GTK_WINDOW(win), da, &geometry, GDK_HINT_RESIZE_INC | GDK_HINT_BASE_SIZE | GDK_HINT_MIN_SIZE);

   gtk_widget_show(win);

   if(!quit) gtk_main();

   if(pd.paintapi) {
      printf("Freeing paintapi\n");
      pd.paintapi->api_free(pd.paintapi);
   }

   return 0;
}

/*
 * Local variables:
 * c-file-style: "ellemtel"
 * c-file-offsets: ((c . c-lineup-dont-change) (statement-cont . (lambda (le) (if (save-excursion (goto-char (cdr le)) (looking-at "return")) (c-lineup-java-inher le) (c-lineup-math le)))))
 * End:
 */
