/*
 * Plot - plot time-based data on screen and to file with interactive controls
 * Copyright (C) 2006  Jonas Berlin <xkr47@outerspace.dyndns.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****
 * eventhandlers-plot-mouse - gtk event handlers related handling
 * mouse events for the plot
 */

#include <gtk/gtktogglebutton.h>
#include <math.h>

#include "eventhandlers-plot-mouse.xh"

#if 0
#define pprintf printf
#else
#define pprintf(...)
#endif

gboolean plot_button_press_event(GtkWidget *widget, GdkEventButton *event, my_plot_view_t *view) {
   int x = event->x;
   int y = event->y;

   // make sure we receive keyboard events if we are clicked.. dunno
   // why this is needed, maybe because I have this custom handler
   // installed? this function _does_ return 0, which should propagate
   // it further.. :P
   gtk_widget_grab_focus(widget);

   switch((int)event->type) {
      case GDK_2BUTTON_PRESS:
	 break;
      case GDK_BUTTON_PRESS:
	 break;
   }

   return 0;
}

#define LIMIT(v,min,max) do{ if(v<(min)) v=min; else if(v>(max)) v=max; } while(0)

// return true if release accepted.. return value doesn't matter if release_button == 0
boolean_t plot_motion_generic(my_plot_view_t *view, int x, int y, int release_button) {
   const boolean_t tailing = 0;

   if(0) {
      double ox = data_scrollbar_controller_get_page_off(view->parent->dsc_x);
      double oy = data_scrollbar_controller_get_page_off(view->dsc_y);
      double lx = data_scrollbar_controller_get_page_length(view->parent->dsc_x);
      double ly = data_scrollbar_controller_get_page_length(view->dsc_y);

      double hx = 0.5 / (view->draw.plot_xe + 1.0) * lx;
      double hy = 0.5 / (view->draw.plot_yde + 1.0) * ly;
      double mx = x / (view->draw.plot_xe + 1.0) * lx + ox;
      double my = (view->draw.plot_yde - y) / (view->draw.plot_yde + 1.0) * ly + oy;

      printf("%4d , %4d -> %.0lf , %8.3lf  (%.0lf,%8.3lf)-(%.0lf,%8.3lf)\n", x, y, mx, my, mx-hx, my-hy, mx+hx, my+hy);
   }

   pprintf("release = %d\n", release_button);

   return 1;
}

gboolean plot_motion_notify_event(GtkWidget *wid, GdkEventMotion *event, my_plot_view_t *view) {
   int x = event->x;
   int y = event->y;

   plot_motion_generic(view, x, y, 0);

   return 0;
}

gboolean plot_button_release_event(GtkWidget *widget, GdkEventButton *event, my_plot_view_t *view) {
   int x = event->x;
   int y = event->y;

   if(plot_motion_generic(view, x, y, (int)event->button)) {

      // call generic motion routine again, this time in BMODE_OFF and as
      // if the pointer just moved to the same location which lets the
      // routine set up anything that's usually established when no
      // button is pressed and the mouse is moved.
      plot_motion_generic(view, x, y, 0);

      update_cursor(view);
   }

   return 0;
}

gboolean plot_enter_notify_event(GtkWidget *widget, GdkEventCrossing *event, my_plot_view_t *view) {
   int x = event->x;
   int y = event->y;

   plot_motion_generic(view, x, y, 0);

   return 0;
}

gboolean plot_leave_notify_event(GtkWidget *widget, GdkEventCrossing *event, my_plot_view_t *view) {

   return 0;
}

/*
 * Local variables:
 * c-file-style: "ellemtel"
 * c-file-offsets: ((c . c-lineup-dont-change) (statement-cont . (lambda (le) (if (save-excursion (goto-char (cdr le)) (looking-at "return")) (c-lineup-java-inher le) (c-lineup-math le)))))
 * End:
 */
