/*
 * Plot - plot time-based data on screen and to file with interactive controls
 * Copyright (C) 2006  Jonas Berlin <xkr47@outerspace.dyndns.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****
 * eventhandlers - gtk event handlers, mostly calls other methods to
 * do the job
 */

#include <gtk/gtkbutton.h>
#include <gtk/gtkcheckbutton.h>
#include <gtk/gtkcheckmenuitem.h>
#include <gtk/gtkdnd.h>
#include <gtk/gtkmenuitem.h>
#include <gtk/gtkrange.h>
#include <gtk/gtkselection.h>
#include <gtk/gtkspinbutton.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtktreestore.h>
#include <gtk/gtktreeview.h>
#include <gtk/gtkcellrenderertext.h>
#include <gtk/gtkcellrenderertoggle.h>

#include <math.h>

#include "../plot_draw.xh"
#include "../system.xh"
#include "eventhandlers-plot-draw.xh"
#include "eventhandlers-plot-keyboard.xh"
#include "eventhandlers-plot-mouse.xh"
#include "eventhandlers.xh"
#include "../gtk/gtkutil.xh"
#include "plot2-datafiles.xh"
#include "plot2.xh"

#if 0
#define pprintf printf
#else
#define pprintf(...)
#endif

static void draw_minmax_clicked(GtkCheckMenuItem *menuitem, my_plot_t *plot) {
   boolean_t newstate = gtk_check_menu_item_get_active(menuitem);

   if(plot->view[0]->draw.draw_min_max_lines == newstate) return;

   for(int vi=0; vi<plot->view_count; ++vi) {
      plot->view[vi]->draw.draw_min_max_lines = newstate;
   }

   reset_gc(plot);

   plot_need_redraw(plot);
}

static void draw_average_clicked(GtkCheckMenuItem *menuitem, my_plot_t *plot) {
   boolean_t newstate = gtk_check_menu_item_get_active(menuitem);

   if(plot->view[0]->draw.draw_average_line == newstate) return;

   for(int vi=0; vi<plot->view_count; ++vi) {
      plot->view[vi]->draw.draw_average_line = newstate;
   }

   reset_gc(plot);

   plot_need_redraw(plot);
}



static void cb_draw_avg_handler(GtkCheckButton *check, my_plot_t *plot) {
   plot->view[0]->draw.draw_average_line = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
   plot_view_need_redraw(plot->view[0]);
}

static void cb_draw_minmax_handler(GtkCheckButton *check, my_plot_t *plot) {
   plot->view[0]->draw.draw_min_max_lines = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
   plot_view_need_redraw(plot->view[0]);
}

static void cb_type_step_handler(GtkCheckButton *check, my_plot_t *plot) {
   boolean_t step = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
   for(int lsi=0; lsi<plot->view[0]->draw.line_set_count; ++lsi) {
      for(int li=0; li<plot->view[0]->draw.line_set[lsi].line_count; ++li) {
	 plot->view[0]->draw.line_set[lsi].line[li].linetype = step ? TYPE_STEP : TYPE_LINE;
      }
   }
   plot_view_need_redraw(plot->view[0]);
}

static boolean_t updating;

static void update_mm(my_plot_t *plot) {
   double min, max;
   data_scrollbar_controller_get_range(plot->dsc_x, &min, &max);

   double start = gtk_spin_button_get_value(plot->spin_start_time) * 1000.0 + min;
   double end = gtk_spin_button_get_value(plot->spin_end_time) * 1000.0 + min;

   data_scrollbar_controller_set_page_length(plot->dsc_x, end - start);
   data_scrollbar_controller_set_page_off(plot->dsc_x, start);
}

static void spin_start_idx_handler(GtkSpinButton *spin, my_plot_t *plot) {
   if(updating) return;
   updating++;
   double min, max;
   data_scrollbar_controller_get_range(plot->dsc_x, &min, &max);

   int idx = (int)round(gtk_spin_button_get_value(spin));
   if(plot->view[0]->draw.line_set) {
      my_dataset_t *dataset = plot->view[0]->draw.line_set[0].dataset;
      my_time_t time = dataset->time[idx] - min;
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(plot->spin_start_time), time / 1000.0);
      char foo[20];
      snprintf(foo, 20, "%.3lf", time / 1000.0);
      gtk_label_set_text(GTK_LABEL(plot->label_start_time_suffix), foo);
      update_mm(plot);
   }
   updating--;
}

static void spin_start_time_handler(GtkSpinButton *spin, my_plot_t *plot) {
   if(updating) return;
   updating++;
   double min, max;
   data_scrollbar_controller_get_range(plot->dsc_x, &min, &max);

   my_time_t time = (my_time_t)round((gtk_spin_button_get_value(spin) * 1000.0) + min);
   if(plot->view[0]->draw.line_set) {
      my_dataset_t *dataset = plot->view[0]->draw.line_set[0].dataset;
      int idx = my_time_arr_find(dataset->time, time, AFB_LEFT_OR_MATCH);
      int idx2 = my_time_arr_find(dataset->time, time, AFB_RIGHT_OR_MATCH);
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(plot->spin_start_idx), idx);
      my_time_t rtime = (my_time_t)round(dataset->time[idx] - min);
      char foo[20];
      snprintf(foo, 20, "%.3lf%s", rtime / 1000.0, idx2 > idx ? " +" : "");
      gtk_label_set_text(GTK_LABEL(plot->label_start_time_suffix), foo);
      update_mm(plot);
   }
   updating--;
}

static void spin_end_idx_handler(GtkSpinButton *spin, my_plot_t *plot) {
   if(updating) return;
   updating++;
   double min, max;
   data_scrollbar_controller_get_range(plot->dsc_x, &min, &max);

   int idx = (int)round(gtk_spin_button_get_value(spin));
   if(plot->view[0]->draw.line_set) {
      my_dataset_t *dataset = plot->view[0]->draw.line_set[0].dataset;
      my_time_t time = dataset->time[idx] - min;
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(plot->spin_end_time), time / 1000.0);
      char foo[20];
      snprintf(foo, 20, "%.3lf", time / 1000.0);
      gtk_label_set_text(GTK_LABEL(plot->label_end_time_suffix), foo);
      update_mm(plot);
   }
   updating--;
}

static void spin_end_time_handler(GtkSpinButton *spin, my_plot_t *plot) {
   if(updating) return;
   updating++;
   double min, max;
   data_scrollbar_controller_get_range(plot->dsc_x, &min, &max);

   my_time_t time = (my_time_t)round((gtk_spin_button_get_value(spin) * 1000.0) + min);
   if(plot->view[0]->draw.line_set) {
      my_dataset_t *dataset = plot->view[0]->draw.line_set[0].dataset;
      int idx = my_time_arr_find(dataset->time, time, AFB_LEFT_OR_MATCH);
      int idx2 = my_time_arr_find(dataset->time, time, AFB_RIGHT_OR_MATCH);
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(plot->spin_end_idx), idx);
      my_time_t rtime = (my_time_t)round(dataset->time[idx2] - min);
      char foo[20];
      snprintf(foo, 20, "%.3lf%s", rtime / 1000.0, idx2 > idx ? " -" : "");
      gtk_label_set_text(GTK_LABEL(plot->label_end_time_suffix), foo);
      update_mm(plot);
   }
   updating--;
}



static void plot_generic_scrollbar_value_changed(my_plot_t *plot) {
   double xlo, xhi;
   double ylo, yhi;
   data_scrollbar_controller_get_range(plot->dsc_x, &xlo, &xhi);
   data_scrollbar_controller_get_range(plot->view[0]->dsc_y, &ylo, &yhi); // TODO multiview

   for(int vi=0; vi<plot->view_count; ++vi) {
      plot->view[vi]->draw.xstart = (my_time_t)xlo;
      plot->view[vi]->draw.xend = (my_time_t)xhi;
   }
}

static void plot_xscrollbar_value_changed(GtkRange *not_always_provided, my_plot_t *plot) {
   double xlength = data_scrollbar_controller_get_page_length(plot->dsc_x);
   double xoff = data_scrollbar_controller_get_page_off(plot->dsc_x);
   
   for(int vi=0; vi<plot->view_count; ++vi) {
      plot->view[vi]->draw.xmin = xoff;
      plot->view[vi]->draw.xmax = xoff + xlength;
   }

   if(!updating) {
      updating++;

      plot_generic_scrollbar_value_changed(plot);

      double min, max;
      data_scrollbar_controller_get_range(plot->dsc_x, &min, &max);
      double diff = (max-min) / 1000.0;
      gtk_spin_button_set_range(GTK_SPIN_BUTTON(plot->spin_start_time), 0, diff);
      gtk_spin_button_set_range(GTK_SPIN_BUTTON(plot->spin_end_time), 0, diff);
      gtk_spin_button_set_increments(GTK_SPIN_BUTTON(plot->spin_start_time), 0.0001, 0.0001);
      gtk_spin_button_set_increments(GTK_SPIN_BUTTON(plot->spin_end_time), 0.0001, 0.0001);

      gtk_spin_button_set_value(GTK_SPIN_BUTTON(plot->spin_start_time), (xoff - min) / 1000.0);
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(plot->spin_end_time), (xoff + xlength - min) / 1000.0);

      if(plot->view[0]->draw.line_set) {
	 my_dataset_t *dataset = plot->view[0]->draw.line_set[0].dataset;
	 int start_idx = my_time_arr_find(dataset->time, (my_time_t)xoff, AFB_LEFT_OR_MATCH);
	 int end_idx = my_time_arr_find(dataset->time, (my_time_t)(xoff + xlength), AFB_RIGHT_OR_MATCH);

	 gtk_spin_button_set_range(GTK_SPIN_BUTTON(plot->spin_start_idx), 0, dataset->time_count - 1);
	 gtk_spin_button_set_range(GTK_SPIN_BUTTON(plot->spin_end_idx), 0, dataset->time_count - 1);

	 gtk_spin_button_set_value(GTK_SPIN_BUTTON(plot->spin_start_idx), (double)start_idx);
	 gtk_spin_button_set_value(GTK_SPIN_BUTTON(plot->spin_end_idx), (double)end_idx);
      }
      updating--;
   }

   plot_need_redraw(plot);
}

static void plot_yscrollbar_value_changed(GtkRange *not_always_provided, my_plot_view_t *view) {

   double ylength = data_scrollbar_controller_get_page_length(view->dsc_y);
   double yoff = data_scrollbar_controller_get_page_off(view->dsc_y);

   view->draw.ymin = yoff;
   view->draw.ymax = yoff + ylength;

   plot_generic_scrollbar_value_changed(view->parent);

   plot_view_need_redraw(view);
}

static gboolean plot_resized(GtkWidget *widget, GdkEventConfigure *do_not_use, my_plot_view_t *view) {
   char foo[40];
   snprintf(foo, sizeof(foo), "%dx%d", view->da->allocation.width, view->da->allocation.height);
   gtk_label_set_text(GTK_LABEL(view->parent->label_plot_size), foo);
   return 0;
}

void plot_view_connect_signals(my_plot_view_t *view, GladeXML *gladexml) {

   // drawing area

   gtk_widget_add_events(view->da, GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
   g_signal_connect(view->da, "key-press-event", G_CALLBACK(plot_key_event), view);
   g_signal_connect(view->da, "key-release-event", G_CALLBACK(plot_key_event), view);
   g_signal_connect(view->da, "focus-in-event", G_CALLBACK(plot_focus_event_before), view);
   g_signal_connect_after(view->da, "focus-in-event", G_CALLBACK(plot_focus_event_after), view);
   g_signal_connect(view->da, "focus-out-event", G_CALLBACK(plot_focus_event_before), view);
   g_signal_connect_after(view->da, "focus-out-event", G_CALLBACK(plot_focus_event_after), view);

   gtk_widget_add_events(view->da, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
   g_signal_connect(view->da, "enter-notify-event", G_CALLBACK(plot_enter_notify_event), view);
   g_signal_connect(view->da, "leave-notify-event", G_CALLBACK(plot_leave_notify_event), view);

   gtk_widget_add_events(view->da, GDK_POINTER_MOTION_MASK);
   g_signal_connect(view->da, "motion-notify-event", G_CALLBACK(plot_motion_notify_event), view);

   gtk_widget_add_events(view->da, GDK_BUTTON_PRESS_MASK);
   g_signal_connect(view->da, "button-press-event", G_CALLBACK(plot_button_press_event), view);

   gtk_widget_add_events(view->da, GDK_BUTTON_RELEASE_MASK);
   g_signal_connect(view->da, "button-release-event", G_CALLBACK(plot_button_release_event), view);


   g_signal_connect(view->da, "map-event", G_CALLBACK(plot_map_event), view);
   g_signal_connect(view->da, "configure-event", G_CALLBACK(plot_configure_event), view);
   g_signal_connect(view->da, "expose-event", G_CALLBACK(plot_expose_event), view);

   // y scrollbars

   g_signal_connect(view->dsc_y->scroll_off, "value-changed", G_CALLBACK(plot_yscrollbar_value_changed), view);
   g_signal_connect(view->dsc_y->scroll_off, "changed", G_CALLBACK(plot_yscrollbar_value_changed), view);

   plot_yscrollbar_value_changed(NULL, view);

   g_signal_connect(view->da, "map-event", G_CALLBACK(plot_resized), view);
   g_signal_connect(view->da, "configure-event", G_CALLBACK(plot_resized), view);
}

void plot_connect_signals(my_plot_t *plot, const char *gladefile, GladeXML *gladexml) {

   plot->cb_draw_avg = glade_xml_get_widget(gladexml, "cb_draw_avg");
   plot->cb_draw_minmax = glade_xml_get_widget(gladexml, "cb_draw_minmax");
   plot->cb_type_step = glade_xml_get_widget(gladexml, "cb_type_step");
   plot->spin_start_idx = glade_xml_get_widget(gladexml, "spin_start_idx");
   plot->spin_end_idx = glade_xml_get_widget(gladexml, "spin_end_idx");
   plot->spin_start_time = glade_xml_get_widget(gladexml, "spin_start_time");
   plot->spin_end_time = glade_xml_get_widget(gladexml, "spin_end_time");
   plot->label_start_time_suffix = glade_xml_get_widget(gladexml, "label_start_time_suffix");
   plot->label_end_time_suffix = glade_xml_get_widget(gladexml, "label_end_time_suffix");
   plot->label_plot_size = glade_xml_get_widget(gladexml, "label_plot_size");

   g_signal_connect(plot->cb_draw_avg, "toggled", G_CALLBACK(cb_draw_avg_handler), plot);
   g_signal_connect(plot->cb_draw_minmax, "toggled", G_CALLBACK(cb_draw_minmax_handler), plot);
   g_signal_connect(plot->cb_type_step, "toggled", G_CALLBACK(cb_type_step_handler), plot);
   g_signal_connect(plot->spin_start_idx, "value-changed", G_CALLBACK(spin_start_idx_handler), plot);
   g_signal_connect(plot->spin_end_idx, "value-changed", G_CALLBACK(spin_end_idx_handler), plot);
   g_signal_connect(plot->spin_start_time, "value-changed", G_CALLBACK(spin_start_time_handler), plot);
   g_signal_connect(plot->spin_end_time, "value-changed", G_CALLBACK(spin_end_time_handler), plot);

   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(plot->cb_draw_avg), true);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(plot->cb_draw_minmax), true);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(plot->cb_type_step), false);

   //////////////////////////////////

   GdkScreen* screen = gtk_window_get_screen(plot->parent);
   g_signal_connect(screen, "size-changed", G_CALLBACK(screen_size_changed_event), plot);

   // x scrollbars

   g_signal_connect(plot->dsc_x->scroll_off, "value-changed", G_CALLBACK(plot_xscrollbar_value_changed), plot);
   g_signal_connect(plot->dsc_x->scroll_off, "changed", G_CALLBACK(plot_xscrollbar_value_changed), plot);

   plot_xscrollbar_value_changed(NULL, plot);
}

/*
 * Local variables:
 * c-file-style: "ellemtel"
 * c-file-offsets: ((c . c-lineup-dont-change) (statement-cont . (lambda (le) (if (save-excursion (goto-char (cdr le)) (looking-at "return")) (c-lineup-java-inher le) (c-lineup-math le)))))
 * End:
 */
