/*
 * Plot - plot time-based data on screen and to file with interactive controls
 * Copyright (C) 2006  Jonas Berlin <xkr47@outerspace.dyndns.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****
 * plot2-datafiles - functions related to datafile management
 */

#include <gtk/gtktogglebutton.h>
#include <gtk/gtktreestore.h>
#include <gtk/gtktreeview.h>

#include "../system.xh"
#include "plot2-datafiles.xh"

// ##TODO## make local
// ##TODO## the colors suck
paintapi_rgb_t default_line_colors[] = {

   { 0xffff, 0x4444, 0x4444 },
   { 0xffff, 0xc0c0, 0x4040 },
   { 0x4040, 0xffff, 0x4040 },
   { 0x4040, 0xffff, 0xffff },
   { 0x4040, 0x8080, 0xffff },
   { 0xc0c0, 0x4040, 0xffff },
   { 0xc0c0, 0xc0c0, 0xc0c0 },

   { 0x8080, 0x2222, 0x2222 },
   { 0x8080, 0x6060, 0x2020 },
   { 0x2020, 0x8080, 0x2020 },
   { 0x2020, 0x8080, 0x8080 },
   { 0x2020, 0x4040, 0x8080 },
   { 0x6060, 0x2020, 0x8080 },
   { 0x6060, 0x6060, 0x6060 },

   { 0xffff, 0xffff, 0xffff },
};

int num_default_line_colors = SZ(default_line_colors);

static int linecoloroff = 0;

void add_datafile(my_plot_t *plot, char *filename) {
   printf(_c("Loading %s .."), filename);
   puts("");

   my_dataset_t *dataset = read_dataset(filename, NULL, plot);

   if(!dataset) {
      if(!quit) {
	 fprintf(stderr, _c("\033[31;1mWARNING: File '%s' could not be loaded\033[m\n"), filename);
      }
      return;
   }

   //printf("maxdiff %Ld min_time_diff %Ld\n", dataset->maxdiff, dataset->min_time_diff);

   int lsi = plot->view[0]->draw.line_set_count;
   for(int vi=0; vi<plot->view_count; ++vi) {
      my_plot_view_t *view = plot->view[vi];

      if(!view->draw.time_off || view->draw.time_off > dataset->ts) {
	 view->draw.time_off = dataset->ts;
      }

      my_plot_line_info_set_arr_set_length(view->draw.line_set, lsi + 1);

      view->draw.line_set[lsi].dataset = dataset;

      for(int dsi=0; dsi<dataset->item_count; ++dsi) {
	 int li = view->draw.line_set[lsi].line_count;
	 my_plot_line_info_arr_set_length(view->draw.line_set[lsi].line, li+1);
	 int linecoloridx = linecoloroff + dsi;
	 view->draw.line_set[lsi].line[li].color = default_line_colors[linecoloridx % SZ(default_line_colors)];
	 view->draw.line_set[lsi].line[li].dataset_idx = dsi;
	 view->draw.line_set[lsi].line[li].enabled = 1;
	 view->draw.line_set[lsi].line[li].mode = LINE_MODE_NORMAL;
      }
   }

   linecoloroff += dataset->item_count;

   plot_need_redraw(plot);
   update_minmax(plot);
   for(int vi=0; vi<plot->view_count; ++vi) {
      show_all(plot->view[vi]);
   }

   if(lsi == 0) {
      // set initial center
      double min, max;
      for(int vi=0; vi<plot->view_count; ++vi) {
	 data_scrollbar_controller_get_range(plot->view[vi]->dsc_y, &min, &max);
	 data_scrollbar_controller_set_zoompoint(plot->view[vi]->dsc_y, min > 0 ? 0 : max < 0 ? 1 : -min / (max - min)); // align to(wards) zero
      }
      data_scrollbar_controller_set_zoompoint(plot->dsc_x, 1.0); // align to right
   }
}

void remove_data_file(my_plot_t *plot, int idx) {
   for(int vi=0; vi<plot->view_count; ++vi) {
      plot_draw_remove_data_file(&plot->view[vi]->draw, idx);
   }

   plot_need_redraw(plot);
   update_minmax(plot);
}

void remove_all_data_files(my_plot_t *plot) {
   for(int vi=0; vi<plot->view_count; ++vi) {
      plot_draw_remove_all_data_files(&plot->view[vi]->draw);
   }

   plot_need_redraw(plot);
   update_minmax(plot);
}

void refresh_all_files(my_plot_t *plot) {
   for(int lsi=0; lsi<plot->view[0]->draw.line_set_count; ++lsi) {
      update_dataset(plot->view[0]->draw.line_set[lsi].dataset);
   }

   plot_need_redraw(plot);
   update_minmax(plot);
}

/*
 * Local variables:
 * c-file-style: "ellemtel"
 * c-file-offsets: ((c . c-lineup-dont-change) (statement-cont . (lambda (le) (if (save-excursion (goto-char (cdr le)) (looking-at "return")) (c-lineup-java-inher le) (c-lineup-math le)))))
 * End:
 */
